from unittest import TestCase
from uw_libraries.dao import MyLib_DAO, SubjectGuide_DAO


class TestDao(TestCase):

    def test_mylib_dao(self):
        self.assertEqual(MyLib_DAO().service_name(), "libraries")

    def test_subjectguide_dao(self):
        self.assertEqual(SubjectGuide_DAO().service_name(), "libcurrics")



"""
This the interface for interacting with the UW Libraries Subject Guide
Web Service.
"""

import json
from urllib.parse import quote
from restclients_core.exceptions import DataFailureException
from uw_libraries.dao import SubjectGuide_DAO
from uw_libraries.models import (
    SubjectGuide, CourseGuide, Library, Librarian)

SubjectGuideDao = SubjectGuide_DAO()
subject_guide_url_prefix = '/currics_db/api/v1/data'


def get_subject_guide_for_section_params(
        year, quarter, curriculum_abbr, course_number, section_id=None):
    """
    Returns a SubjectGuide model for the passed section params:

    year: year for the section term (4-digits)
    quarter: quarter (AUT, WIN, SPR, or SUM)
    curriculum_abbr: curriculum abbreviation
    course_number: course number
    section_id: course section identifier (optional)
    """

    quarter = quarter.upper()[:3]

    url = "{}/{}/{}/{}/{}/{}/{}".format(
        subject_guide_url_prefix, 'course', year, quarter,
        quote(curriculum_abbr.upper()), course_number, section_id.upper())
    headers = {'Accept': 'application/json'}

    response = SubjectGuideDao.getURL(url, headers)
    response_data = str(response.data)
    if response.status != 200:
        raise DataFailureException(url, response.status, response_data)

    return _subject_guide_from_json(json.loads(response.data))


def get_subject_guide_for_section(section):
    """
    Returns a SubjectGuide model for the passed SWS section model.
    """
    return get_subject_guide_for_section_params(
        section.term.year, section.term.quarter, section.curriculum_abbr,
        section.course_number, section.section_id)


def get_subject_guide_for_canvas_course_sis_id(course_sis_id):
    """
    Returns a SubjectGuide model for the passed Canvas course SIS ID.
    """
    (year, quarter, curriculum_abbr, course_number,
        section_id) = course_sis_id.split('-', 4)
    return get_subject_guide_for_section_params(
        year, quarter, curriculum_abbr, course_number, section_id)


def get_default_subject_guide(campus='seattle'):
    """
    Returns a default SubjectGuide model for the passed campus:
        seattle, bothell, tacoma
    """
    url = "{}/{}/{}".format(subject_guide_url_prefix, 'defaultGuide', campus)
    headers = {'Accept': 'application/json'}

    response = SubjectGuideDao.getURL(url, headers)

    if response.status != 200:
        raise DataFailureException(url, response.status, response.data)

    data = json.loads(response.data)
    return _subject_guide_from_json(data)


def _subject_guide_from_json(data):
    subject_data = data['subjectGuide']
    subject_guide = SubjectGuide()
    subject_guide.contact_url = subject_data.get('askUsLink', None)
    subject_guide.contact_text = subject_data.get('askUsText', None)
    subject_guide.discipline = subject_data.get('discipline', None)
    subject_guide.find_librarian_url = subject_data.get(
        'findLibrarianLink', None)
    subject_guide.find_librarian_text = subject_data.get(
        'findLibrarianText', None)
    subject_guide.guide_url = subject_data.get('guideLink', None)
    subject_guide.guide_text = subject_data.get('guideText', None)
    subject_guide.faq_url = subject_data.get('howDoILink', None)
    subject_guide.faq_text = subject_data.get('howDoIText', None)
    subject_guide.writing_guide_url = subject_data.get(
        'writingGuideLink', None)
    subject_guide.writing_guide_text = subject_data.get(
        'writingGuideText', None)

    default_guide_campus = subject_data.get('defaultGuideForCampus', None)
    if default_guide_campus is not None:
        subject_guide.is_default_guide = True
        subject_guide.default_guide_campus = default_guide_campus
    else:
        subject_guide.is_default_guide = False

    subject_guide.libraries = []
    subject_guide.librarians = []

    for libdata in subject_data.get('libraries', []):
        library = Library()
        library.name = libdata.get('name', None)
        library.description = libdata.get('description', None)
        library.url = libdata.get('url', None)
        subject_guide.libraries.append(library)

    for libdata in subject_data.get('librarians', []):
        librarian = Librarian()
        librarian.name = libdata.get('name', None)
        librarian.email = libdata.get('email', None)
        librarian.phone = libdata.get('telephone', None)
        librarian.url = libdata.get('url', None)
        subject_guide.librarians.append(librarian)

    if 'courseGuide' in data:
        course_data = data['courseGuide']
        subject_guide.course_guide = CourseGuide()
        subject_guide.course_guide.guide_url = course_data.get(
            'guideLink', None)
        subject_guide.course_guide.guide_text = course_data.get(
            'guideText', None)

    return subject_guide



from unittest import TestCase
from uw_libraries.mylib import get_account, get_account_html
from uw_libraries.util import fdao_mylib_override
from restclients_core.exceptions import DataFailureException
from datetime import date


@fdao_mylib_override
class MyLibInfoTest(TestCase):
    def test_get_account(self):
        account = get_account("javerage")
        self.assertEquals(account.holds_ready, 1)
        self.assertEquals(account.fines, 5.35)
        self.assertEquals(account.items_loaned, 3)
        self.assertEquals(account.get_next_due_date_str(),
                          "2020-10-15T02:00:00+00:00")
        self.assertIsNotNone(str(account))
        self.assertEquals(
            account.json_data(),
            {'holds_ready': 1,
             'fines': 5.35,
             'items_loaned': 3,
             'next_due': '2020-10-15T02:00:00+00:00'})

        account = get_account("jnewstudent")
        self.assertIsNone(account.next_due)
        self.assertEquals(account.holds_ready, 0)
        self.assertEquals(account.fines, 0.0)
        self.assertEquals(account.items_loaned, 0)

    def test_html_response(self):
        response = get_account_html("javerage")
        self.assertEquals(response, (
            b'<p>You have 7 items checked out.<br>\nYou have items '
            b'due back on 2014-04-29.<br>\nYou don\'t owe any fines.</p>\n<a '
            b'href="http://alliance-primo.hosted.exlibrisgroup.com/'
            b'primo_library/libweb/action/dlBasketGet.do?vid=UW&redirectTo='
            b'myAccount">Go to your account</a>'))

    def test_bad_json(self):
        self.assertRaises(Exception, get_account, "badjsonuser")

        try:
            get_account("badjsonuser")
            self.assertTrue(False, "Shouldn't get here")
        except Exception as ex:
            self.assertTrue("example bad data" in str(ex))

    def test_invalid_user(self):
        # Testing error message in a 200 response
        self.assertRaises(DataFailureException, get_account, "invalidnetid")
        # Testing non-200 response
        self.assertRaises(DataFailureException, get_account, "invalidnetid123")

        try:
            get_account("invalidnetid")
        except DataFailureException as ex:
            self.assertTrue("User not found" in str(ex.msg))

    def test_with_timestamp(self):
        response = get_account_html('javerage', timestamp=1391122522900)
        self.assertEquals(response, (
            b'<p>You have 7 items checked out.<br>\n You have items '
            b'due back on 2014-04-29.<br>\n You don\'t owe any fines.</p>\n '
            b'<a href="http://alliance-primo.hosted.exlibrisgroup.com/'
            b'primo_library/libweb/action/dlBasketGet.do?vid=UW&amp;'
            b'redirectTo=myAccount">Go to your account</a>'))



"""
This is the interface for interacting with the UW Libraries Web Service.
"""

from datetime import datetime
from dateutil.parser import parse
import json
from urllib.parse import urlencode
from uw_libraries.dao import MyLib_DAO
from uw_libraries.models import MyLibAccount
from restclients_core.exceptions import DataFailureException

MyLibDao = MyLib_DAO()
mylib_url_prefix = '/mylibinfo/v1/'


def _get_mylib_resource(url):
    response = MyLibDao.getURL(url, {})

    response_data = str(response.data)
    if response.status != 200:
        raise DataFailureException(url, response.status, response_data)

    # 'Bug' with lib API causing requests with no/invalid user to return a 200
    if "User not found" in response_data:
        raise DataFailureException(url, 404, response_data)

    return response.data


def _get_account(netid, timestamp=None, is_html=False):
    params = [("id", netid)]
    if timestamp is not None:
        params.append(("timestamp", timestamp))
    if is_html:
        params.append(("style", "html"))

    url = "{}?{}".format(mylib_url_prefix, urlencode(params))
    return _get_mylib_resource(url)


def get_account(netid, timestamp=None):
    """
    The Libraries object has a method for getting information
    about a user's library account
    """
    response = _get_account(netid, timestamp=timestamp)
    return _account_from_json(response)


def get_account_html(netid, timestamp=None):
    """
    The Libraries object has a method for getting information
    about a user's library account
    """
    return _get_account(netid, timestamp=timestamp, is_html=True)


def _account_from_json(body):
    try:
        account_data = json.loads(body)
    except Exception as ex:
        raise Exception(
            "Unable to parse mylibinfo: {}. {}".format(body, ex))
    account = MyLibAccount()
    account.fines = account_data["fines"]
    account.holds_ready = account_data["holds_ready"]
    account.items_loaned = account_data["items_loaned"]
    account.next_due = account_data.get("next_due_utc")
    if account.next_due is not None:
        account.next_due = parse(account.next_due)
    return account



"""
Contains UW Library DAO implementations.
"""
from restclients_core.dao import DAO
from os.path import abspath, dirname
import os


class MyLib_DAO(DAO):
    def service_name(self):
        return "libraries"

    def service_mock_paths(self):
        return [abspath(os.path.join(dirname(__file__), "resources"))]


class SubjectGuide_DAO(DAO):
    def service_name(self):
        return "libcurrics"

    def service_mock_paths(self):
        return [abspath(os.path.join(dirname(__file__), "resources"))]



from unittest import TestCase
from uw_libraries.subject_guides import (
    get_subject_guide_for_section_params, get_subject_guide_for_section,
    get_subject_guide_for_canvas_course_sis_id, get_default_subject_guide)
from uw_libraries.util import fdao_subject_guide_override
from restclients_core.exceptions import DataFailureException
from uw_sws.models import Section, Term
from datetime import date


@fdao_subject_guide_override
class SubjectGuideTest(TestCase):
    def test_subject_guide_with_bad_section_params(self):
        # Valid curriculum, with no file
        self.assertRaises(DataFailureException,
                          get_subject_guide_for_section_params,
                          year=2012, quarter='aut', curriculum_abbr='B ARTS',
                          course_number='197', section_id='A')

        # Missing params
        self.assertRaises(TypeError, get_subject_guide_for_section_params)

        # URL capitalization and quoting
        try:
            # Using a non-existant section so we can inspect the url
            guide = get_subject_guide_for_section_params(
                year=1990, quarter='aut', curriculum_abbr='A B&C',
                course_number='101', section_id='a')
        except DataFailureException as ex:
            self.assertEquals(
                ex.url,
                '/currics_db/api/v1/data/course/1990/AUT/A%20B%26C/101/A',
                'Quoted curriculum abbr')

    def test_subject_guide_for_section_params(self):
        guide = get_subject_guide_for_section_params(
            year=2015, quarter='aut', curriculum_abbr='MATH',
            course_number='309', section_id='A')

        self.assertEquals(guide.discipline, 'Mathematics')
        self.assertEquals(
            guide.contact_url, 'http://www.lib.washington.edu/about/contact')
        self.assertEquals(
            guide.find_librarian_url,
            'http://guides.lib.uw.edu/research/subject-librarians')
        self.assertEquals(
            guide.guide_url,
            'http://guides.lib.uw.edu/friendly.php?s=research/math')
        self.assertEquals(
            guide.faq_url, 'http://guides.lib.uw.edu/research/faq')
        self.assertEquals(guide.is_default_guide, False)
        self.assertEquals(len(guide.libraries), 1)
        self.assertEquals(
            guide.libraries[0].name, 'Mathematics Research Library')
        self.assertEquals(
            guide.libraries[0].url, 'http://www.lib.washington.edu/math')
        self.assertEquals(len(guide.librarians), 2)
        self.assertEquals(guide.librarians[0].email, 'javerage@uw.edu')
        self.assertEquals(guide.librarians[0].name, 'J Average')
        self.assertEquals(
            guide.librarians[0].url,
            'http://guides.lib.washington.edu/Javerage')
        self.assertEquals(guide.librarians[1].email, 'baverage@uw.edu')
        self.assertEquals(guide.librarians[1].name, 'B Average')
        self.assertEquals(
            guide.librarians[1].url,
            'http://guides.lib.washington.edu/Baverage')
        self.assertEquals(
            guide.course_guide.guide_url,
            'http://guides.lib.uw.edu/seattle/math307smith')
        self.assertEquals(
            guide.course_guide.guide_text,
            'MATH 307: Introduction To Differential Equations (Smith)')

    def test_subject_guide_for_section(self):
        term = Term(year=2015, quarter='autumn')
        section = Section(term=term, curriculum_abbr='MATH',
                          course_number='309', section_id='A')

        guide = get_subject_guide_for_section(section)

        self.assertEquals(guide.discipline, 'Mathematics')
        self.assertEquals(
            guide.contact_url, 'http://www.lib.washington.edu/about/contact')
        self.assertEquals(
            guide.find_librarian_url,
            'http://guides.lib.uw.edu/research/subject-librarians')
        self.assertEquals(
            guide.guide_url,
            'http://guides.lib.uw.edu/friendly.php?s=research/math')
        self.assertEquals(
            guide.faq_url, 'http://guides.lib.uw.edu/research/faq')
        self.assertEquals(guide.is_default_guide, False)
        self.assertEquals(len(guide.libraries), 1)
        self.assertEquals(
            guide.libraries[0].name, 'Mathematics Research Library')
        self.assertEquals(
            guide.libraries[0].url, 'http://www.lib.washington.edu/math')
        self.assertEquals(len(guide.librarians), 2)
        self.assertEquals(guide.librarians[0].email, 'javerage@uw.edu')
        self.assertEquals(guide.librarians[0].name, 'J Average')
        self.assertEquals(
            guide.librarians[0].url,
            'http://guides.lib.washington.edu/Javerage')
        self.assertEquals(guide.librarians[1].email, 'baverage@uw.edu')
        self.assertEquals(guide.librarians[1].name, 'B Average')
        self.assertEquals(
            guide.librarians[1].url,
            'http://guides.lib.washington.edu/Baverage')
        self.assertEquals(
            guide.course_guide.guide_url,
            'http://guides.lib.uw.edu/seattle/math307smith')
        self.assertEquals(
            guide.course_guide.guide_text,
            'MATH 307: Introduction To Differential Equations (Smith)')

    def test_subject_guide_for_canvas_course_sis_id(self):
        sis_id = '2015-autumn-MATH-309-A'
        guide = get_subject_guide_for_canvas_course_sis_id(sis_id)

        self.assertEquals(guide.discipline, 'Mathematics')
        self.assertEquals(
            guide.contact_url, 'http://www.lib.washington.edu/about/contact')
        self.assertEquals(
            guide.find_librarian_url,
            'http://guides.lib.uw.edu/research/subject-librarians')
        self.assertEquals(
            guide.guide_url,
            'http://guides.lib.uw.edu/friendly.php?s=research/math')
        self.assertEquals(
            guide.faq_url, 'http://guides.lib.uw.edu/research/faq')
        self.assertEquals(guide.is_default_guide, False)
        self.assertEquals(len(guide.libraries), 1)
        self.assertEquals(
            guide.libraries[0].name, 'Mathematics Research Library')
        self.assertEquals(
            guide.libraries[0].url, 'http://www.lib.washington.edu/math')
        self.assertEquals(len(guide.librarians), 2)
        self.assertEquals(guide.librarians[0].email, 'javerage@uw.edu')
        self.assertEquals(guide.librarians[0].name, 'J Average')
        self.assertEquals(
            guide.librarians[0].url,
            'http://guides.lib.washington.edu/Javerage')
        self.assertEquals(guide.librarians[1].email, 'baverage@uw.edu')
        self.assertEquals(guide.librarians[1].name, 'B Average')
        self.assertEquals(
            guide.librarians[1].url,
            'http://guides.lib.washington.edu/Baverage')
        self.assertEquals(
            guide.course_guide.guide_url,
            'http://guides.lib.uw.edu/seattle/math307smith')
        self.assertEquals(
            guide.course_guide.guide_text,
            'MATH 307: Introduction To Differential Equations (Smith)')

    def test_default_subject_guide(self):
        guide = get_default_subject_guide(campus='tacoma')

        self.assertEquals(guide.discipline, 'your discipline')
        self.assertEquals(
            guide.contact_url, 'http://www.tacoma.uw.edu/library/contact-us')
        self.assertEquals(
            guide.find_librarian_url,
            'http://www.tacoma.uw.edu/library/subject-librarians')
        self.assertEquals(guide.guide_url, 'http://guides.lib.uw.edu/tacoma')
        self.assertEquals(guide.faq_url, 'http://guides.lib.uw.edu/research')
        self.assertEquals(guide.is_default_guide, True)
        self.assertEquals(guide.default_guide_campus, 'tacoma')
        self.assertEquals(len(guide.libraries), 1)
        self.assertEquals(guide.libraries[0].name, 'UW Tacoma Library')
        self.assertEquals(
            guide.libraries[0].url, 'http://www.tacoma.uw.edu/library')



from commonconf import override_settings


fdao_mylib_override = override_settings(
    RESTCLIENTS_LIBRARIES_DAO_CLASS='Mock')

fdao_subject_guide_override = override_settings(
    RESTCLIENTS_LIBCURRICS_DAO_CLASS='Mock')



import json
from restclients_core import models


class Library(models.Model):
    name = models.CharField(max_length=250, null=True)
    description = models.CharField(max_length=1000, null=True)
    url = models.CharField(max_length=250, null=True)


class Librarian(models.Model):
    name = models.CharField(max_length=250, null=True)
    email = models.CharField(max_length=100, null=True)
    phone = models.CharField(max_length=20, null=True)
    url = models.CharField(max_length=250, null=True)


class CourseGuide(models.Model):
    guide_url = models.CharField(max_length=250, null=True)
    guide_text = models.CharField(max_length=1000, null=True)


class SubjectGuide(models.Model):
    contact_url = models.CharField(max_length=250, null=True)
    contact_text = models.CharField(max_length=1000, null=True)
    discipline = models.CharField(max_length=250, null=True)
    find_librarian_url = models.CharField(max_length=250, null=True)
    find_librarian_text = models.CharField(max_length=1000, null=True)
    guide_url = models.CharField(max_length=250, null=True)
    guide_text = models.CharField(max_length=1000, null=True)
    faq_url = models.CharField(max_length=250, null=True)
    faq_text = models.CharField(max_length=1000, null=True)
    writing_guide_url = models.CharField(max_length=250, null=True)
    writing_guide_text = models.CharField(max_length=1000, null=True)
    is_default_guide = models.NullBooleanField()
    default_guide_campus = models.CharField(max_length=50, null=True)
    course_guide = models.ForeignKey(CourseGuide, null=True)


class MyLibAccount(models.Model):
    holds_ready = models.IntegerField()
    fines = models.DecimalField(max_digits=8, decimal_places=2)
    items_loaned = models.IntegerField()
    next_due = models.DateTimeField(null=True)

    def get_next_due_date_str(self):
        """
        return next due date in the ISO format (yyyy-mm-dd).
        If the next_due is None, return None.
        """
        if self.next_due is not None:
            return self.next_due.isoformat()  # 2020-07-15T02:00:00+00:00
        return None

    def json_data(self):
        return {'holds_ready': self.holds_ready,
                'fines': self.fines,
                'items_loaned': self.items_loaned,
                'next_due': self.get_next_due_date_str()
                }

    def __str__(self):
        return json.dumps(self.json_data())
